library flutter_sdk;

import 'package:flutter/cupertino.dart';
import 'dart:async';
import 'package:flutter/material.dart';
import 'dart:ui';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:flutter_sdk/values/color.dart';
import 'package:flutter_sdk/values/textstyle.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';


class SwipeRouteBuilder extends PageRouteBuilder {

  SwipeRouteBuilder(Widget previousPage, Widget page, {bool fromLeft = false})
      :super(
      pageBuilder: (context, animation, secondaryAnimation) =>
          DirectionalSlideTransition(
              CurvedAnimation(parent: animation, curve: Curves.easeInOut),
              _fromLeft,
              page
          ),
      transitionDuration: Duration(milliseconds: 500),
      reverseTransitionDuration: Duration(milliseconds: 500)
  ) {
    _fromLeft.value = fromLeft;
  }

}

ValueNotifier<bool> _fromLeft = ValueNotifier(true);



class IllegalRouteException implements Exception {
  final message;
  IllegalRouteException(this.message);
}

abstract class DisposableAnimatedWidget extends StatefulWidget {
  /// Creates a widget that rebuilds when the given listenable changes.
  ///
  /// The [listenable] argument is required.
  DisposableAnimatedWidget({
    Key? key,
    required this.listenable,
  }) : super(key: key);

  /// The [Listenable] to which this widget is listening.
  ///
  /// Commonly an [Animation] or a [ChangeNotifier].
  final Listenable listenable;


  Widget build(BuildContext context);

  void dispose();

  /// Subclasses typically do not override this method.
  @override
  _AnimatedState createState() => _AnimatedState();


}

class _AnimatedState extends State<DisposableAnimatedWidget> {
  @override
  void initState() {
    super.initState();
    widget.listenable.addListener(_handleChange);
  }

  @override
  void didUpdateWidget(DisposableAnimatedWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.listenable != oldWidget.listenable) {
      oldWidget.listenable.removeListener(_handleChange);
      widget.listenable.addListener(_handleChange);
    }
  }

  @override
  void dispose() {
    widget.listenable.removeListener(_handleChange);
    widget.dispose();
    super.dispose();
  }

  void _handleChange() {
    setState(() {
      // The listenable's state is our build state, and it changed already.
    });
  }

  @override
  Widget build(BuildContext context) => widget.build(context);
}

class DirectionalSlideTransition extends DisposableAnimatedWidget{
  DirectionalSlideTransition(
      Animation<double> animation,
      this.fromLeft,
      this.child
      ) : super(listenable: animation){

    _forwardOffset = fromLeft.value ? _leftOffsetBuilder : _rightOffsetBuilder;
    _backwardOffset = fromLeft.value ? _rightOffsetBuilder : _leftOffsetBuilder;
    _offsetBuilder = () => _forwardOffset();

    _animationListener = (AnimationStatus status){
      if(status == AnimationStatus.forward){
        _offsetBuilder = () => _forwardOffset();
      }else{
        _offsetBuilder = () => _backwardOffset();
      }
    };

    _fromLeftListener = (){
      if(fromLeft.value){
        _forwardOffset = _leftOffsetBuilder;
        _backwardOffset = _rightOffsetBuilder;
      }else{
        _forwardOffset = _rightOffsetBuilder;
        _backwardOffset = _leftOffsetBuilder;
      }
    };

    this.animation.addStatusListener(_animationListener);

    fromLeft.addListener(_fromLeftListener);

  }

  final ValueNotifier<bool> fromLeft;
  final Widget child;

  late final Function() _fromLeftListener;
  late final Function(AnimationStatus) _animationListener;

  Animation<double> get animation => listenable as Animation<double>;

  Offset _leftOffsetBuilder(){
    return Offset(-1+this.animation.value, 0);
  }

  Offset _rightOffsetBuilder(){
    return Offset(1-this.animation.value, 0);
  }

  late Offset Function() _forwardOffset;
  late Offset Function() _backwardOffset;

  late Offset Function() _offsetBuilder;

  @override
  Widget build(BuildContext context) {
    Offset offset = _offsetBuilder();
    return FractionalTranslation(
      translation: offset,
      child: child,
    );
  }

  @override
  void dispose() {
    fromLeft.removeListener(_fromLeftListener);
    animation.removeStatusListener(_animationListener);
  }

}

class BottomMenu{
  final Widget icon;
  final String title;
  final Widget screen;
  final String routeName;
  final int index;

  const BottomMenu({required this.title, required this.icon, required this.routeName, required this.screen, required this.index});
}


abstract class BaseText extends Text{
  BaseText(
      String text, {
        required Color color,
        FontWeight? fontWeight,
        double? fontSize})
      : super.rich(
      TextSpan(
          text: text,
          style: TextStyle(
              color: color,
              fontWeight: fontWeight,
              fontSize: fontSize
          )
      )
  );

  Size get size =>  (TextPainter(text: textSpan, textDirection: TextDirection.ltr)..layout(minWidth: 0, maxWidth: double.infinity)).size;
}

class TitleText extends BaseText{
  TitleText(
      String text, {
        Color? color,
        FontWeight? fontWeight,
        double? fontSize,
        TextStyle? style
      }) : super(
      text,
      color: color??ZEColor.Primary,
      fontWeight: fontWeight??style?.fontWeight??heading4.fontWeight,
      fontSize: fontSize??style?.fontSize??heading4.fontSize
  );
}

class BodyText extends BaseText{
  BodyText(
      String text, {
        Color? color,
        FontWeight? fontWeight,
        double? fontSize,
        TextStyle? style
      }) : super(
      text,
      color: color??ZEColor.Primary[variant]!,
      fontWeight: fontWeight??style?.fontWeight,
      fontSize: fontSize??style?.fontSize??para1.fontSize);
}

class HintText extends BaseText{
  HintText(
      String text, {
        Color? color,
        FontWeight? fontWeight,
        double? fontSize,
        TextStyle? style
      }) : super(
      text,
      color: color??ZEColor.Primary[shade]!,
      fontWeight: fontWeight??style?.fontWeight,
      fontSize: fontSize??style?.fontSize??para1.fontSize
  );
}

class TitleTextWithController extends HookWidget{
  final Color? color;
  final FontWeight? fontWeight;
  final double? fontSize;
  final ValueNotifier controller;
  final TextStyle? style;
  TitleTextWithController(this.controller, {this.style, this.color, this.fontWeight, this.fontSize});

  @override
  Widget build(BuildContext context) {
    final _controller = useProvider(ChangeNotifierProvider((ref)=>controller));

    return _controller.value!=null
        ? TitleText(
        _controller.value!.toString(),
        color:color,
        fontWeight: fontWeight,
        fontSize: fontSize,
        style: style)
        : SizedBox();
  }
}

class BodyTextWithController extends HookWidget{
  final Color? color;
  final FontWeight? fontWeight;
  final double? fontSize;
  final ValueNotifier controller;
  final TextStyle? style;
  BodyTextWithController(this.controller, {this.color, this.style, this.fontWeight, this.fontSize});

  @override
  Widget build(BuildContext context) {
    final _controller = useProvider(ChangeNotifierProvider((ref)=>controller));
    return _controller.value!=null
        ? BodyText(
        _controller.value!.toString(),
        color:color,
        fontWeight: fontWeight,
        fontSize: fontSize,
        style: style)
        : SizedBox();
  }
}

class HintTextWithController extends HookWidget{
  final Color? color;
  final FontWeight? fontWeight;
  final double? fontSize;
  final TextStyle? style;
  final ValueNotifier controller;
  HintTextWithController(this.controller, {this.style, this.color, this.fontWeight, this.fontSize});

  @override
  Widget build(BuildContext context) {
    final _controller = useProvider(ChangeNotifierProvider((ref)=>controller));
    return _controller.value!=null
        ? HintText(
        _controller.value!.toString(),
        color:color,
        fontWeight: fontWeight,
        fontSize: fontSize,
        style: style)
        : SizedBox();
  }
}

class ZEInputField extends StatelessWidget {
  final String? title;
  final String? hint;
  final TextInputType? inputType;
  final TextEditingController? controller;
  final bool readOnly;
  ZEInputField({this.title, this.hint, this.inputType, this.controller, this.readOnly=false});

  @override
  Widget build(BuildContext context) {

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        title!=null?Padding(
            padding: const EdgeInsets.only(bottom: 10),
            child:TitleText(title!, style: heading6)
        ):SizedBox(),
        TextFormField(
          readOnly: readOnly,
          keyboardType: inputType,
          controller: controller,
          cursorColor: ZEColor.Pink,
          decoration: InputDecoration(
              hintText: hint,
              hintStyle: TextStyle(color: ZEColor.Primary[shade]),
              alignLabelWithHint: true,
              focusedBorder: OutlineInputBorder(borderSide: BorderSide(color: ZEColor.Pink, width: 2), borderRadius: BorderRadius.circular(10)),
              enabledBorder: OutlineInputBorder(borderSide: BorderSide(color: ZEColor.Background[lightShade]!), borderRadius: BorderRadius.circular(10))
          ),
        )
      ],
    );
  }

}

class ZEDropDown extends StatelessWidget{
  final String? title;
  final String? hint;
  final List<String> items;
  final ListController<String> controller;


  ZEDropDown({this.title, this.hint, required this.items, required this.controller});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        title!=null?Padding(
            padding: const EdgeInsets.only(bottom: 10),
            child:TitleText(title!, style: heading6)
        ):SizedBox(),
        DropDown(controller: controller, items: items, hint: hint,)
      ],
    );
  }

}

class DropDown extends HookWidget{

  final String? hint;
  final List<String> items;
  final ListController<String> controller;
  final InputBorder? focusBorder;
  final InputBorder? enabledBBorder;
  final bool isSearchable;

  DropDown({this.hint, required this.items, required this.controller, this.focusBorder, this.enabledBBorder, this.isSearchable = false});



  @override
  Widget build(BuildContext context) {
    final textEditingController = useTextEditingControllerWithListController(controller);
    return TextField(
      onTap: () {
        showDialog(context: context, builder: (context) {
          return _CustomDialogList(isSearchable, items, controller);
        });
      },
      readOnly: true,
      controller: textEditingController,
      decoration: InputDecoration(
          hintText: hint,
          hintStyle: TextStyle(color: ZEColor.Primary[shade]),
          alignLabelWithHint: true,
          focusedBorder: focusBorder??OutlineInputBorder(borderSide: BorderSide(color: ZEColor.Pink, width: 2), borderRadius: BorderRadius.circular(10)),
          enabledBorder: enabledBBorder??OutlineInputBorder(borderSide: BorderSide(color: ZEColor.Background[lightShade]!), borderRadius: BorderRadius.circular(10))
      ),
    );
  }

}

final _searchController = TextEditingController();

class _CustomDialogList extends StatelessWidget{
  final bool isSearchable;
  final List<String> items;
  final ListController<String> controller;


  _CustomDialogList(this.isSearchable, this.items, this.controller);

  @override
  Widget build(BuildContext context) {
    final height = MediaQuery.of(context).size.height;

    return Dialog(
        child: Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              isSearchable ? Container(
                  decoration: BoxDecoration(
                      border: Border(bottom: BorderSide(color: ZEColor.Primary[shade]!))
                  ),
                  child:Row(
                    children: [
                      Padding(padding: EdgeInsets.symmetric(horizontal: 5), child: Icon(Icons.search)),
                      Expanded(child: TextField(
                        controller: _searchController,
                        decoration: InputDecoration(
                            border: InputBorder.none
                        ),
                      ))
                    ],
                  )): Container(),
              Container(
                constraints: BoxConstraints(
                    maxHeight: height * 0.4
                ),
                child:
                _CustomListView(items: items, controller: controller, searchController: _searchController,),
              )
            ]
        )
    );
  }
}

class _CustomListView extends HookWidget{
  final List<String> items;
  final ListController controller;
  final TextEditingController searchController;
  _CustomListView({required this.items, required this.controller, required this.searchController});


  @override
  Widget build(BuildContext context) {
    final listNotifier = useProvider(ChangeNotifierProvider((ref)=>controller));
    final listController = useListControllerWithSearchController(searchController, items);

    return Scrollbar(child:ListView(
        shrinkWrap: true,
        children: listController.list.map((i)=> InkWell(
            onTap: (){
              if(listNotifier.list.contains(i)){
                listNotifier.remove(i);
              }else{
                listNotifier.add(i);
              }
            },
            child:
            Row(children: [
              Expanded(flex: 2, child: Padding(padding: EdgeInsets.all(15), child:Text(i))),
              Expanded(flex:1, child:listNotifier.list.contains(i) ? Icon(Icons.check): SizedBox())
            ],))
        ).toList()
    ));
  }

}



class ZECheckBox extends HookWidget{
  final ValueNotifier<bool> controller;
  final Color? activeColor;
  ZECheckBox(this.controller, {this.activeColor});

  @override
  Widget build(BuildContext context) {
    final checkState = useProvider(ChangeNotifierProvider((ref)=>controller));
    return Checkbox(
      value: checkState.value,
      onChanged: (value){
        if(value!=null) controller.value = value;
      },
      activeColor: activeColor??ZEColor.Green,
    );
  }
}

class ZEButton extends StatelessWidget{

  final String text;
  final double width;
  final Color? color;
  final Function()? onTap;
  ZEButton(this.text, {this.width = 100, this.color = ZEColor.Green, this.onTap});

  @override
  Widget build(BuildContext context) {


    return InkWell(
        onTap: onTap ,
        child:Container(
            width: width,
            child: Padding(
                padding: const EdgeInsets.symmetric(vertical: 18),
                child:Center(
                    child:TitleText(
                        text,
                        color: ZEColor.White,
                        style: heading5
                    ))),
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(8),
                color: color
            )
        ));
  }
}

class AddButton extends StatelessWidget {
  final Function()? onTap;
  final double radius;
  AddButton({this.onTap, required this.radius});

  @override
  Widget build(BuildContext context) {
    return InkWell(
        onTap: onTap,
        child: Container(
          width: radius*2,
          height: radius*2,
          decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(radius),
              color: ZEColor.Green
          ),
          child: Center(
              child:Icon(
                Icons.add,
                color: ZEColor.White,
              )),)
    );
  }
}

TextEditingController useTextEditingControllerWithListController(ListController<String> listController){
  return use(_DropDownHook(listController));
}

class _DropDownHook extends Hook<TextEditingController>{
  final ListController<String> listController;

  _DropDownHook(this.listController);

  @override
  HookState<TextEditingController, Hook<TextEditingController>> createState() {
    return _DropDownHookState();
  }

}

class _DropDownHookState extends HookState<TextEditingController, _DropDownHook>{
  final TextEditingController _textEditingController = TextEditingController();

  late Function() listener;

  @override
  void initHook() {
    super.initHook();
    _textEditingController.text = hook.listController.list.toString().substring(1, hook.listController.list.toString().length-1);
    listener  = () {
      _textEditingController.text = hook.listController.list.toString().substring(1, hook.listController.list.toString().length-1);
    };
    hook.listController.addListener(listener);
  }

  @override
  TextEditingController build(BuildContext context) {
    return _textEditingController;

  }

  @override
  void dispose() {
    super.dispose();
    hook.listController.removeListener(listener);
  }

}

ListController<String> useListControllerWithSearchController(TextEditingController searchController, List<String> items){
  return use(_ListControllerWithSearchController(searchController, items));
}

class _ListControllerWithSearchController extends Hook<ListController<String>> {
  final TextEditingController searchController;
  final List<String> items;

  _ListControllerWithSearchController(this.searchController, this.items);

  @override
  _ListControllerWithSearchControllerState createState() => _ListControllerWithSearchControllerState();
}

class _ListControllerWithSearchControllerState extends HookState<ListController<String>, _ListControllerWithSearchController> {
  late final ListController<String> listController = ListController<String>();

  late Function() listener;

  @override
  void initHook() {
    listController.addAll(hook.items);

    listener = () {
      listController.clear();
      listController.list.addAll(hook.items.where((i) {
        return (i.toLowerCase().contains(hook.searchController.text.toLowerCase()));
      }));
      print(listController.list);
    };

    hook.searchController.addListener(listener);

    listController.addListener(() {
      setState(() { });
    });
    super.initHook();
  }

  @override
  void dispose() {
    hook.searchController.removeListener(listener);
    listController.removeListener(() {
      setState(() { });
    });
    super.dispose();
  }

  @override
  ListController<String> build(BuildContext context) {
    return listController;
  }
}


ValueNotifier<bool> useIsListEmptyNotifierWithListController(ListController<String> listController){
  return use(_CheckEnabledNotifierHook(listController));
}


class _CheckEnabledNotifierHook extends Hook<ValueNotifier<bool>> {
  final ListController<String> listController;
  _CheckEnabledNotifierHook(this.listController);

  @override
  _CheckEnabledNotifierHookState createState() => _CheckEnabledNotifierHookState();
}

class _CheckEnabledNotifierHookState extends HookState<ValueNotifier<bool>, _CheckEnabledNotifierHook> {
  final valueNotifier = ValueNotifier(false);

  late Function() listener;

  @override
  void initHook() {
    valueNotifier.value = hook.listController.list.isNotEmpty;
    listener = (){
      valueNotifier.value = hook.listController.list.isNotEmpty;
    };

    hook.listController.addListener(listener);

    valueNotifier.addListener(() {
      setState(() {});
    });
    super.initHook();
  }

  @override
  ValueNotifier<bool> build(BuildContext context) {
    return valueNotifier;
  }

  @override
  void dispose() {
    hook.listController.removeListener(listener);
    valueNotifier.removeListener(() {
      setState(() { });
    });
    super.dispose();
  }
}


class TopDialogRoute extends OverlayRoute{
  final Widget child;
  TopDialogRoute(this.child);


  @override
  Iterable<OverlayEntry> createOverlayEntries() {
    final List<OverlayEntry> overlayEntries = [];
    overlayEntries.add(
      OverlayEntry(builder: (context){
        return Semantics(
          child:child,
          focused: false,
          container: true,
          explicitChildNodes: true,
        );
      },
        opaque: false,
      ),

    );
    return overlayEntries;
  }

}

class Dialog extends StatelessWidget {
  /// Creates a dialog.
  ///
  /// Typically used in conjunction with [showDialog].
  const Dialog({
    Key? key,
    this.backgroundColor,
    this.elevation,
    this.insetAnimationDuration = const Duration(milliseconds: 100),
    this.insetAnimationCurve = Curves.decelerate,
    this.insetPadding,
    this.clipBehavior = Clip.none,
    this.shape,
    this.child,
  }) : assert(clipBehavior != null),
        super(key: key);

  /// {@template flutter.material.dialog.backgroundColor}
  /// The background color of the surface of this [Dialog].
  ///
  /// This sets the [Material.color] on this [Dialog]'s [Material].
  ///
  /// If `null`, [ThemeData.dialogBackgroundColor] is used.
  /// {@endtemplate}
  final Color? backgroundColor;

  /// {@template flutter.material.dialog.elevation}
  /// The z-coordinate of this [Dialog].
  ///
  /// If null then [DialogTheme.elevation] is used, and if that's null then the
  /// dialog's elevation is 24.0.
  /// {@endtemplate}
  /// {@macro flutter.material.material.elevation}
  final double? elevation;

  /// {@template flutter.material.dialog.insetAnimationDuration}
  /// The duration of the animation to show when the system keyboard intrudes
  /// into the space that the dialog is placed in.
  ///
  /// Defaults to 100 milliseconds.
  /// {@endtemplate}
  final Duration insetAnimationDuration;

  /// {@template flutter.material.dialog.insetAnimationCurve}
  /// The curve to use for the animation shown when the system keyboard intrudes
  /// into the space that the dialog is placed in.
  ///
  /// Defaults to [Curves.decelerate].
  /// {@endtemplate}
  final Curve insetAnimationCurve;

  /// {@template flutter.material.dialog.insetPadding}
  /// The amount of padding added to [MediaQueryData.viewInsets] on the outside
  /// of the dialog. This defines the minimum space between the screen's edges
  /// and the dialog.
  ///
  /// Defaults to `EdgeInsets.symmetric(horizontal: 40.0, vertical: 24.0)`.
  /// {@endtemplate}
  final EdgeInsets? insetPadding;

  /// {@template flutter.material.dialog.clipBehavior}
  /// Controls how the contents of the dialog are clipped (or not) to the given
  /// [shape].
  ///
  /// See the enum [Clip] for details of all possible options and their common
  /// use cases.
  ///
  /// Defaults to [Clip.none], and must not be null.
  /// {@endtemplate}
  final Clip clipBehavior;

  /// {@template flutter.material.dialog.shape}
  /// The shape of this dialog's border.
  ///
  /// Defines the dialog's [Material.shape].
  ///
  /// The default shape is a [RoundedRectangleBorder] with a radius of 4.0
  /// {@endtemplate}
  final ShapeBorder? shape;

  /// The widget below this widget in the tree.
  ///
  /// {@macro flutter.widgets.ProxyWidget.child}
  final Widget? child;

  static const RoundedRectangleBorder _defaultDialogShape =
  RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(4.0)));
  static const double _defaultElevation = 24.0;

  @override
  Widget build(BuildContext context) {
    final DialogTheme dialogTheme = DialogTheme.of(context);
    final EdgeInsets effectivePadding = MediaQuery.of(context).viewInsets + (insetPadding ?? EdgeInsets.zero);
    return AnimatedPadding(
      padding: effectivePadding,
      duration: insetAnimationDuration,
      curve: insetAnimationCurve,
      child: MediaQuery.removeViewInsets(
        removeLeft: true,
        removeTop: true,
        removeRight: true,
        removeBottom: true,
        context: context,
        child: Align(
          alignment: Alignment.topCenter,
          child: ConstrainedBox(
            constraints: const BoxConstraints(minWidth: 280.0),
            child: Material(
              color: backgroundColor ?? dialogTheme.backgroundColor ?? Theme.of(context).dialogBackgroundColor,
              elevation: elevation ?? dialogTheme.elevation ?? _defaultElevation,
              shape: shape ?? dialogTheme.shape ?? _defaultDialogShape,
              type: MaterialType.card,
              clipBehavior: clipBehavior,
              child: child,
            ),
          ),
        ),
      ),
    );
  }
}


class DialogService {
  DialogService._();

  static late DialogService _getDialogService = DialogService._();

  factory DialogService(){
    return _getDialogService;
  }

  late Function _showDialog;
  Completer _completer = Completer();

  void registerDialogListener(Function f){
    _showDialog = f;
  }

  Future openDialog({required String message, required Color color, Widget? icon}) async{
    _completer = Completer();
    _showDialog(message: message, color: color, icon: icon);
    return _completer.future;
  }

  void closeDialog(){
    _completer.complete();
  }

}

class ActionHolder{
  ActionHolder({required this.title, required this.onPressed});
  final String title;
  final Function onPressed;
}

class ListController<T> extends ChangeNotifier{
  ListController([List<T>? list]){
    if(list!=null) {
      _list = List.from(list);
    }
  }

  List<T> _list = <T>[];
  List<T> get list => _list;

  addAll(List<T> list){
    _list.addAll(list);
    notifyListeners();
  }

  add(T value){
    _list.add(value);
    notifyListeners();
  }

  remove(T value){
    _list.remove(value);
    notifyListeners();
  }

  clear(){
    _list.clear();
    notifyListeners();
  }

  set list(List<T> newList){
    _list = newList;
    notifyListeners();
  }

}



class DialogManager extends StatefulWidget{
  final Widget child;
  DialogManager({Key? key, required this.child}):super(key: key);
  @override
  State<StatefulWidget> createState() => _DialogManagerState();

}

class _DialogManagerState extends State<DialogManager> with TickerProviderStateMixin {
  final DialogService dialogService = DialogService();

  bool _dialogPresent = false;
  Timer? timer;

  late double width = MediaQuery
      .of(context)
      .size
      .width;

  @override
  void initState() {
    dialogService.registerDialogListener(dialog);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return widget.child;
  }

  Future dialog(
      {required String message, required Color color, Widget? icon}) async {
    showTopDialog(
        context: context,
        transitionBuilder: (context, animation, secondaryAnimation, child) =>
            SlideTransition(
              position: Tween<Offset>(
                  begin: Offset(0, -0.06),
                  end: Offset.zero
              ).animate(
                  CurvedAnimation(parent: animation, curve: Curves.decelerate)),
              child: FadeTransition(
                opacity: CurvedAnimation(
                    curve: Curves.easeOut,
                    parent: animation
                ),
                child: child,
              ),),
        transitionDuration: Duration(milliseconds: 250),
        pageBuilder: (BuildContext context, Animation<double> animation,
            Animation<double> secondaryAnimation) {
          return Dialog(
              elevation: 0,
              backgroundColor: Color(0x00000000),
              insetPadding: EdgeInsets.only(top: MediaQuery
                  .of(context)
                  .padding
                  .top + 15,),
              shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(10)),
              child: Dismissible(
                key: Key(context.toString()),
                onDismissed: (direction){
                  if(_dialogPresent) {
                    _closeDialog();
                  }
                },
                child:Container(
                  decoration: BoxDecoration(
                      color: color,
                      borderRadius: BorderRadius.circular(10)
                  ),
                  width: width * 0.9,
                  child: Row(
                    children: [
                      Expanded(flex: 6,
                          child: Padding(
                              padding: EdgeInsets.symmetric(horizontal: 10),
                              child: icon ?? SizedBox())),
                      Expanded(flex: 30,
                          child: Padding(
                              padding: EdgeInsets.symmetric(vertical: 20),
                              child: TitleText(
                                message, color: ZEColor.White, fontSize: 16,))),
                      Expanded(flex: 5,
                          child: InkWell(
                              onTap: () {
                                _closeDialog();
                              },
                              child: Container(height: 32, width: 32,
                                decoration: BoxDecoration(shape: BoxShape.circle,
                                    color: Color(0x33ffffff)),
                                child: Center(
                                    child: Icon(Icons.close, color: ZEColor.White)
                                ),
                              ))),
                      Expanded(flex: 1, child: SizedBox(),)
                    ],
                  ),
                ),
              ));
        });
    dialogService.closeDialog();
  }

  late AnimationController animation;

  showTopDialog({
    required BuildContext context,
    required Widget Function(BuildContext context, Animation<
        double> animation, Animation<double> secondaryAnimation) pageBuilder,
    required Widget Function(BuildContext context, Animation<
        double> animation, Animation<
        double> secondaryAnimation, Widget child) transitionBuilder,
    required Duration transitionDuration
  }) {
    animation = AnimationController(
        vsync: this, duration: transitionDuration);
    final AnimationController secondaryAnimation = AnimationController(
        vsync: this, duration: transitionDuration);
    animation.forward();
    final dialog = pageBuilder.call(context, animation, secondaryAnimation);
    final child = transitionBuilder.call(context, animation, secondaryAnimation, dialog);
    if(_dialogPresent) {
      Navigator.of(context).pushReplacement(TopDialogRoute(child));
    }else{
      Navigator.of(context).push(TopDialogRoute(child));
      _dialogPresent = true;
    }
    timer?.cancel();
    timer = Timer(Duration(seconds: 3), (){
      if(_dialogPresent) {
        _closeDialog();
      }
    });
  }

  void _closeDialog(){
    timer?.cancel();
    animation.reverse().then((value) =>
        Navigator.of(context).pop(context)
    );

    _dialogPresent = false;
  }
}


